@@ -1,6 +1,9 @@
 /*
  *	Copyright 1988 by Rayan S. Zachariassen, all rights reserved.
  *	This will be free software, but only when it is finished.
+ *
+ *	Feature maintenance by  Matti Aarnio <mea@nic.funet.fi> 1991-1996
+ *
  */
 
 /* Sendmail -- a sendmail compatible interface to ZMailer */
@@ -71,6 +74,7 @@ main(argc, argv)
 	char	*LC_ctype;
 	char	buf[8192];
 	char   *buf2 = NULL;
+	char	*bodytype = NULL;
 
 	if ((cp = strrchr(argv[0], '/')) != NULL)
 		progname = ++cp;
@@ -101,6 +105,7 @@ main(argc, argv)
 		case 'B':
 			/* Sendmail 8.7 compability:
 			   -B8BITMIME */
+			bodytype = optarg;
 			break;
 		case 'J': break; /* Sony NEWS OS  JIS-conversion option,
 				    ignore */
@@ -245,10 +250,21 @@ otherprog:
 		mailshare = MAILSHARE;
 	path = NULL;
 	if (speaksmtp) {
+		char *argv[30+1];
 		path = emalloc((unsigned)(strlen(mailbin)+1+strlen(SMTPSERVER)+1));
 		sprintf(path, "%s/%s", mailbin, SMTPSERVER);
+		argv[0] = "smtp-in";
+		argv[1] = "-i";
+		n = 2;
+		cp = getzenv("SMTPOPTIONS"); /* Normal smtp-server options */
 		/* pass on suidness */
-		execl(path, "smtp-in", "-i", (char *)NULL);
+		s = strtok(cp, " \t\"'");
+		do {
+		  argv[n++] = s;
+		  s = strtok(NULL, " \t\"'");
+		} while (s != NULL && n < 30);
+		argv[n] = NULL;
+		execv(path, argv);
 		perror(path);
 	} else if (daemon_flg) {
 		setuid(getuid());
@@ -395,6 +411,8 @@ otherprog:
 		}
 		if (external)
 		  fprintf(mfp, "external\n");
+		if (bodytype && *bodytype != 0)
+		  fprintf(mfp, "bodytype %s\n", bodytype);
 		if (fullname != NULL && *fullname != '\0')
 		  /* maybe we should put it in the environment? */
 		  fprintf(mfp, "fullname %s\n", fullname);
