@@ -166,7 +166,7 @@ reg_comp (str)
 			progname, str);
 		return (NULL);
 	}
-	bzero((void*)prog, sizeof(regexp));
+	memset((void*)prog, 0, sizeof(regexp));
 
 	prog->pattern = str;
 
@@ -1039,7 +1039,7 @@ tsetsubexps(sepp, tre)
 		sep = (struct si_retab *)tmalloc(sizeof (struct si_retab));
 		for (i=0; i < (sizeof sep->startp)/(sizeof sep->startp[0]);++i)
 			sep->startp[i] = sep->endp[i] = NULL;
-		bzero((char *)sep, sizeof (struct si_retab));
+		memset((char *)sep, 0, sizeof (struct si_retab));
 		sep->trep = tre;
 		sep->next = *sepp;
 	} else if (psep != NULL) {
@@ -1065,7 +1065,7 @@ setsubexps(sepp, prog)
 			break;
 	if (sep == NULL) {
 		sep = (struct si_retab *)tmalloc(sizeof (struct si_retab));
-		bzero((char *)sep, sizeof (struct si_retab));
+		memset((char *)sep, 0, sizeof (struct si_retab));
 		if (prog != NULL)
 			sep->match = (char **) tmalloc((prog->re.re_nsub+1) *
 						       sizeof(char *));
