@@ -285,7 +285,7 @@ sslw_error(errCode)
 		putchar(' ');
 	putchar('v');
 	putchar('\n');
-	printf("\t%*.*s\n", cp - bos, cp - bos, bos);
+	printf("\t%*.*s\n", (int)(cp - bos), (int)(cp - bos), bos);
 
 	/* all errors are fatal because its almost impossible to recover well */
 	processing = 0;
