@@ -252,9 +252,13 @@ extern char *VersionNumb;
 extern u_char *rfc822date __((time_t *));
 static int mvdata __((FILE *, char *));
 static int mvbdata __((FILE *, char *, long, int *));
-extern void type __(());
-extern void debug_report __(());
-static void help __(());
+#if defined(HAVE_STDARG_H) && defined(HAVE_VPRINTF)
+extern void type __((int code, char *fmt, ...));
+#else
+extern void type __((/* int code, char *fmt, ... */));
+#endif
+extern void debug_report __((int, char *, char *));
+static void help __((struct smtpconf *, char *));
 extern time_t time __((time_t *));
 static char *router __((char*, int, char*));
 static int partridge __((char*, struct smtpconf *));
@@ -457,8 +461,7 @@ main(argc, argv)
 	if (!daemon_flg) {
 	  strcpy(rhostname, "stdin");
 	  ihostaddr[0] = '\0';
-	  if (ident_flag)
-	    sprintf(ident_username, "uid#%d", getuid());
+	  sprintf(ident_username, "uid#%d@localhost", getuid());
 	  smtpserver(0);
 	} else
 #ifdef	USE_INET
@@ -475,34 +478,34 @@ main(argc, argv)
 	      exit(1);
 	    }
 	    setrhostname(&raddr);
-	    if (ident_flag != 0) {
 #ifdef LOG_MAIL
 #ifdef  LOG_PID
-	      openlog("smtpserver", LOG_PID|LOG_CONS, LOG_MAIL);
+	    openlog("smtpserver", LOG_PID|LOG_CONS, LOG_MAIL);
 #else  /* !LOG_PID */
-	      openlog("smtpserver", 0);
+	    openlog("smtpserver", 0);
 #endif /* LOG_PID */
 #else
-	      openlog("smtpserver", 0);
+	    openlog("smtpserver", 0);
 #endif /* LOG_MAIL */
+
+	    if (ident_flag != 0) {
 	      setrfc1413ident(0);
-#ifdef	LOG_INFO
-	      syslog(LOG_INFO,
-		     "connection from %s@%s (port %d)",
-		     ident_username, rhostname, rport);
-#endif /* LOG_INFO */
-	      pid = getpid();
-	      if (!logfp)
-		openlogfp(daemon_flg);
-	      if (logfp != NULL)
-		fprintf(logfp,
-			"%d#\tconnection from %s@%s (port %d)\n",
-			pid, ident_username, rhostname,
-			rport);
-	      if (ident_flag)
-		sprintf(ident_username+strlen(ident_username),
-			"@%s (port %d)",rhostname, rport);
+	    } else {
+	      strcpy(ident_username, "IDENT-NOT-QUERIED");
 	    }
+	    sprintf(ident_username+strlen(ident_username),
+		    "@%s (port %d)",rhostname, rport);
+#ifdef	LOG_INFO
+	    syslog(LOG_INFO, "connection from %s\n", ident_username);
+#endif /* LOG_INFO */
+
+	    pid = getpid();
+	    if (!logfp)
+	      openlogfp(daemon_flg);
+	    if (logfp != NULL)
+	      fprintf(logfp, "%d#\tconnection from %s\n",
+		      pid, ident_username);
+	      
 #if 0
 	    SIGNAL_HANDLE(SIGCHLD, SIG_DFL);
 #else
@@ -562,25 +565,17 @@ main(argc, argv)
 	      exit(1);
 	    }
 	    settrusteduser();	/* dig out the trusted user ID */
+	    closelog();		/* close the syslog too.. */
 	    detach();		/* this must NOT close fd's */
-	    
+	    /* Close fd's 0, 1, 2 now */
+	    close(0); close(1); close(2);
+
+	    open("/dev/null", O_RDWR, 0);
+	    dup(0);
+	    dup(0); /* fd's 0, 1, 2 are in use again.. */
+
 	    if (!port_set || port != htons(25))
 	      killprevious(0, PID_SMTPSERVER); /* deposit pid */
-#if 1
-	    close(0); close(1); close(2);
-	    open("/dev/null",O_RDONLY);
-	    open("/dev/null",O_RDONLY);
-#else
-	    for (msgfd = getdtablesize() - 1; msgfd >= 0; --msgfd)
-	      if (msgfd != s) {
-		/* junk any buffered output... */
-		if (msgfd == FILENO(stdout))
-		  fflush(stdout);
-		else if (msgfd == FILENO(stderr))
-		  fflush(stderr);
-		close(msgfd);
-	      }
-#endif
 #if 1
 	    pid = getpid();
 	    if (!logfp)
@@ -653,37 +648,38 @@ main(argc, argv)
 		}
 
 		SIGNAL_HANDLE(SIGTERM, SIG_IGN);
+
 		rport = raddr.sin_port;
 		setrhostname(&raddr);
 
-		if (ident_flag != 0) {
 #ifdef  LOG_MAIL
 #ifdef  LOG_PID
-		  openlog("smtpserver", LOG_PID|LOG_CONS, LOG_MAIL);
+		openlog("smtpserver", LOG_PID|LOG_CONS, LOG_MAIL);
 #else  /* !LOG_PID */
-		  openlog("smtpserver", 0);
+		openlog("smtpserver", 0);
 #endif /* LOG_PID */
 #else  /* !LOG_MAIL */
-		  openlog("smtpserver", 0);
+		openlog("smtpserver", 0);
 #endif /* !LOG_MAIL */
+
+		if (ident_flag != 0) {
 		  setrfc1413ident(0);
-#ifdef	LOG_INFO
-		  syslog(LOG_INFO,
-			 "connection from %s@%s (port %d)",
-			 ident_username, rhostname, rport);
-#endif /* LOG_INFO */
-		  pid = getpid();
-		  if (!logfp)
-		    openlogfp(1);
-		  if (logfp != NULL)
-		    fprintf(logfp,
-			    "%d#\tconnection from %s@%s (port %d)\n",
-			    pid, ident_username, rhostname,
-			    rport);
-		  if (ident_flag)
-		    sprintf(ident_username+strlen(ident_username),
-			    "@%s (port %d)",rhostname,rport);
+		} else {
+		  strcpy(ident_username, "IDENT-NOT-QUERIED");
 		}
+		sprintf(ident_username+strlen(ident_username),
+			"@%s (port %d)", rhostname, rport);
+#ifdef	LOG_INFO
+		syslog(LOG_INFO, "connection from %s", ident_username);
+#endif /* LOG_INFO */
+
+		pid = getpid();
+
+		if (!logfp)
+		  openlogfp(1);
+		if (logfp != NULL)
+		  fprintf(logfp,
+			  "%d#\tconnection from %s\n", pid, ident_username);
 
 /* if (logfp) fprintf(logfp,"%d#\tInput fd=%d\n",getpid(),msgfd); */
 		
@@ -1004,6 +1000,7 @@ int insecure;
 	char *s;
 	int rc;
 	char *drpt_envid;
+	char *bodytype = NULL;
 
 	/* For ESMTP SIZE-option use we need to know
 	   how much space we have, it is easiest by
@@ -1085,15 +1082,25 @@ int insecure;
 	while (*s) {
 	  while (*s && (*s == ' '||*s == '\t')) ++s;
 	  if (CISTREQN("BODY=",s,5)) {
-	    /* Actually we do not use this data. */
+	    /* Actually we do not use this data... */
 	    s += 5;
+	    if (bodytype != NULL) {
+	      type(501,"BODY= double definition!");
+	      rc = 1;
+	      break;
+	    }
 	    if (CISTREQN(s,"8BITMIME",8)) {
+	      bodytype = "8BITMIME";
 	      s += 8;
+	    } else if (CISTREQN(s,"BINARYMIME",10)) {
+	      bodytype = "BINARYMIME";
+	      s += 10;
 	    } else if (CISTREQN(s,"7BIT",4)) {
+	      bodytype = "7BIT";
 	      s += 4;
 	    }
 	    if (*s && *s != ' ' && *s != '\t') {
-	      type(455,"BODY-param data error, must be either 8BITMIME, or 7BIT");
+	      type(455,"BODY-param data error, must be one of: 8BITMIME/BINARYMIME/7BIT");
 	      rc = 1;
 	      break;
 	    }
@@ -1179,11 +1186,13 @@ int insecure;
 		  helobuf, ihostaddr);
 	else
 	  fprintf(mfp, "rcvdfrom %s\n", helobuf);
+	if (bodytype != NULL)
+	  fprintf(mfp, "bodytype %s\n", bodytype);
 	fprintf(mfp, "with %s\n", with_protocol);
 	if (ident_flag)
 	  fprintf(mfp, "identinfo %s\n", ident_username);
 	if (*cp == '\0')
-	  fprintf(mfp, "channel error\n");
+	  fprintf(mfp, "channel error\n"); /* Empty  MAIL FROM:<> */
 	else
 	  fprintf(mfp, "from <%s>\n", cp);
 	if (drpt_envid != NULL)
@@ -1706,10 +1715,7 @@ smtpserver(insecure)
 	    strcpy(myhostname,hostent->h_name);
 	}
 
-	type(220, "%s ZMailer Server %s ESMTP%s ready at %s",
-	     myhostname, VersionNumb, ident_flag ? "+IDENT":"", cp);
-	fflush(stdout);
-
+	/* re-opening the log ?? */
 #ifdef	LOG_MAIL
 #ifdef	LOG_PID
 	openlog("smtpserver", LOG_PID|LOG_CONS, LOG_MAIL);
@@ -1720,6 +1726,22 @@ smtpserver(insecure)
 	openlog("smtpserver", 0);
 #endif	/* LOG_MAIL */
 
+#ifdef HAVE_TCPD_H /* TCP-Wrapper code */
+	if (wantconn(s_inputfd, "smtp-receiver") == 0) {
+	  syslog(LOG_WARNING, "refusing connection from %s/%s",
+		 rhostname, ident_username);
+	  type(421, "%s ZMailer Server %s WILL NOT TALK TO YOU at %s",
+	       myhostname, VersionNumb, cp);
+	  fflush(stdout);
+	  sleep(1);
+	  exit(0);
+	}
+#endif
+
+	type(220, "%s ZMailer Server %s ESMTP%s ready at %s",
+	     myhostname, VersionNumb, ident_flag ? "+IDENT":"", cp);
+	fflush(stdout);
+
 	state = Hello;
 	if ((!insecure
 	     || (ihostaddr[0] != '\0' && strcmp(ihostaddr, "[127.0.0.1]") == 0))
@@ -2477,12 +2499,25 @@ router(function, holdlast, args)
 }
 
 
+#ifdef HAVE_VPRINTF
+#ifdef HAVE_STDARG_H
+void
+type(int code, char *fmt, ...)
+#else
+/* VARARGS2 */
+void
+type(code, fmt, va_alist)
+	int code;
+	char *fmt;
+	va_dcl
+#endif
+#else
 /* VARARGS2 */
-
 void
 type(code, fmt, s1, s2, s3, s4, s5, s6)
 	int code;
 	char *fmt, *s1, *s2, *s3, *s4, *s5, *s6;
+#endif
 {
 	char format[BUFSIZ];
 	char *text = NULL;
@@ -2569,6 +2604,30 @@ type(code, fmt, s1, s2, s3, s4, s5, s6)
 		text = "Transaction failed";
 		break;
 	}
+#ifdef HAVE_VPRINTF
+	{
+	va_list ap;
+#ifdef HAVE_STDARG_H
+	va_start(ap,fmt);
+#else
+	va_start(ap);
+#endif
+	if (fmt != NULL)
+		vprintf(fmt,  ap);
+	else
+		vprintf(text, ap);
+	printf("\r\n");
+	if (logfp != NULL) {
+		if (fmt != NULL)
+			vfprintf(logfp, fmt,  ap);
+		else
+			vfprintf(logfp, text, ap);
+		fprintf(logfp, "\n");
+		fflush(logfp);
+	}
+	va_end(ap);
+	}
+#else
 	if (fmt != NULL)
 		printf(fmt,  s1, s2, s3, s4, s5, s6);
 	else
@@ -2582,6 +2641,7 @@ type(code, fmt, s1, s2, s3, s4, s5, s6)
 		fprintf(logfp, "\n");
 		fflush(logfp);
 	}
+#endif
 }
 
 
