@@ -1138,7 +1138,9 @@ putmail(dp, rp, fdmail, fdopmode, timestring, file)
 	  fputs("\001\001\001\001\n",fp);
 
 	fromuser = rp->addr->link->user;
-	if (*fromuser == 0) fromuser = "postmaster";
+	if (*fromuser == 0 ||
+	    strcmp(rp->addr->link->channel, "error") == 0)
+	  fromuser = "postmaster";
 	if (fprintf(fp, "%s%s  %s",
 		    FROM_,fromuser,timestring) == EOF
 	    || writeheaders(rp,fp,"\n",convert_qp,0) < 0) {
@@ -1832,7 +1834,7 @@ biff(hostname, username, offset)
 	if (spl == NULL) {
 	  if ((hp = gethostbyname(hostname)) == NULL)
 	    return;
-	  bzero((char *)&biffaddr, sizeof biffaddr);
+	  memset((char *)&biffaddr, 0, sizeof biffaddr);
 	  biffaddr.sin_family = hp->h_addrtype;
 	  hp_init(hp);
 	  memcpy((char *)&biffaddr.sin_addr, *hp_getaddr(),  hp->h_length);
